@@ -8,9 +8,9 @@ from datetime import datetime
 frozen = False
 root = ""
 app_name = "scNodes"
-version = "1.0.63"
+version = "1.0.64"
 license = "GNU GPL v3"
-logpath = "log.txt"
+logpath = "scNodes.log"
 nodes = list()
 active_node = None
 focused_node = None
@@ -94,6 +94,7 @@ def set_error(error_object, error_message):
     error_logged = False
 
 def save_project(filename):
+    ## TODO change to work with new json export mode
     if filename[-5:] == '.srnp':
         filename = filename[:-5]
     with open(filename+'.srnp', 'wb') as pickle_file:
