@@ -42,8 +42,11 @@ class Dataset:
             parts[1::2] = map(int, parts[1::2])
             return parts
 
-        # stack
         img = Image.open(self.path)
+        tif = tifffile.TiffFile(self.path)
+        n_frames = len(tif.pages)
+        print(n_frames)
+        print(img.n_frames)
         if img.n_frames > 1:
             for i in range(img.n_frames):
                 self.n_frames += 1
@@ -131,6 +134,10 @@ class Frame:
         self.index = index
         self.framenr = framenr
         self.path = path
+        if "/" in self.path:
+            self.title = self.path[self.path[:-1].rfind("/"):]
+        else:
+            self.title = self.path
         self.data = None
         self.raw_data = None
         self.width = None
@@ -149,30 +156,26 @@ class Frame:
     def load(self):
         if self.data is not None:
             return self.data
-        elif self.raw_data is not None:
-            self.data = self.raw_data.copy()
-            return self.data
         else:
             # 221202 Note: using tifffile instead of PIL now!
             if self.index is None:
-                self.raw_data = tifffile.imread(self.path).astype(np.float)
+                self.data = tifffile.imread(self.path).astype(np.float)
             else:
-                self.raw_data = tifffile.imread(self.path, key=self.index).astype(np.float)
-            self.data = self.raw_data.copy()
-            self.width = self.raw_data.shape[0]
-            self.height = self.raw_data.shape[1]
+                self.data = tifffile.imread(self.path, key=self.index).astype(np.float)
+            self.width = self.data.shape[0]
+            self.height = self.data.shape[1]
             return self.data
 
     def clean(self):
         self.translation = [0.0, 0.0]
         self.discard = False
-        self.data = None ## TODO: check whether app is faster and output still correct when this line is removed.
+        self.data = None
         self.maxima = list()
 
     def clone(self):
         return copy.deepcopy(self)
 
-    def bake_transform(self, interpolation = 1, edges = 'constant', preserve_range=False):
+    def bake_transform(self, interpolation=1, edges='constant', preserve_range=False):
         tmat = transform.AffineTransform(np.matrix([[1.0, 0.0, self.translation[0]], [0.0, 1.0, self.translation[1]], [0.0, 0.0, 1.0]]))
         self.data = transform.warp(self.data, tmat, order=interpolation, mode=edges, preserve_range=preserve_range)
 
@@ -183,7 +186,7 @@ class Frame:
             return self.load()[roi[1]:roi[3], roi[0]:roi[2]]
 
     def __str__(self):
-        sstr = f"Frame at path: \n{self.path}\n" \
+        sstr = "Frame: "+ self.title + "\n" \
             + ("Discarded frame. " if self.discard else "Frame in use. ") \
             + f"Shift of ({self.translation[0]:.2f}, {self.translation[1]:.2f}) pixels detected."
         if self.maxima is not []:
@@ -298,7 +301,7 @@ class ParticleData:
         :param roi: The ROI used in the initial particle position estimation (e.g. ParticleDetectionNode). It can be saved in ParticleData in order to facilitate overlaying the final reconstruction with the corresponding region of the widefield image.
         :return:
         """
-        print("Setting reconstruction ROI to: ", roi[0], roi[1], roi[2], roi[3]) ## check! todo
+        print("Setting reconstruction ROI to: ", roi[0], roi[1], roi[2], roi[3]) ## 0213, 1032
         self.reconstruction_roi = roi
 
     @staticmethod
@@ -318,6 +321,8 @@ class ParticleData:
         _idx_b = df.columns.get_loc("bkgstd [counts]")
         _idx_f = df.columns.get_loc("frame")
         for i in range(df.shape[0]):
+            if (i % 100000 == 0):
+                print(i)
             _data = df.iloc[i]
             particles.append(Particle(
                 _data[_idx_f],
@@ -334,10 +339,6 @@ class ParticleData:
         particle_data_obj.particles = particles
         particle_data_obj.empty = False
         particle_data_obj.bake()
-        print("uncertainty")
-        print(particle_data_obj.parameter["uncertainty [nm]"])
-        print("sigma")
-        print(particle_data_obj.parameter["sigma [nm]"])
         particle_data_obj.set_reconstruction_roi([particle_data_obj.x_min, particle_data_obj.y_min, particle_data_obj.x_max, particle_data_obj.y_max])
 
         return particle_data_obj
