@@ -186,6 +186,7 @@ class ImageViewer:
             ImageViewer.MARKER_COLOUR = cfg.active_node.colour
             self.node_blocking_frame_window = cfg.active_node.DISABLE_FRAME_INFO_WINDOW
             if self.previous_active_node is not cfg.active_node or cfg.active_node.any_change or self.new_image_requested:
+                print("IV getting new image")
                 if cfg.active_node.NODE_RETURNS_IMAGE:
                     an = cfg.active_node
                     self.current_dataset = an.get_source_load_data_node(an).dataset
@@ -235,9 +236,9 @@ class ImageViewer:
 
         # Brightness/contrast window
         if self.contrast_window_open:
-            imgui.set_next_window_size(ImageViewer.CONTRAST_WINDOW_SIZE[0], ImageViewer.CONTRAST_WINDOW_SIZE[1])
+            imgui.set_next_window_size_constraints((ImageViewer.CONTRAST_WINDOW_SIZE[0], 100), (-1, -1))
             imgui.set_next_window_position(self.contrast_window_position[0], self.contrast_window_position[1], condition = imgui.APPEARING)
-            _, self.contrast_window_open = imgui.begin("Adjust contrast", closable = True, flags = imgui.WINDOW_NO_COLLAPSE | imgui.WINDOW_NO_RESIZE)
+            _, self.contrast_window_open = imgui.begin("Adjust contrast", closable = True, flags = imgui.WINDOW_NO_COLLAPSE | imgui.WINDOW_NO_RESIZE | imgui.WINDOW_ALWAYS_AUTO_RESIZE)
             self.contrast_window_position = imgui.get_window_position()
             available_width = imgui.get_content_region_available_width()
             _rgb_button_spacing = 4
