@@ -4,6 +4,9 @@ import time
 import datetime
 from scNodes.core.dataset import *
 from scNodes.core import settings
+from joblib import Parallel, delayed
+from joblib.externals.loky import set_loky_pickler
+set_loky_pickler("dill")
 
 
 class Node:
@@ -48,13 +51,14 @@ class Node:
         self.position = [0, 0]
         self.last_measured_window_position = [0, 0]
         self.node_height = 100
-        self.connectable_attributes = list()
+        self.connectable_attributes = dict()
         self.play = False
         self.any_change = False
-        self.queued_actions = list()
         self.use_roi = False
         self.roi = [0, 0, 0, 0]
         self.lut = "auto"
+        self.params = dict()
+
         cfg.nodes.append(self)
 
         # some bookkeeping vars - ignore these
@@ -77,14 +81,13 @@ class Node:
         self.OVERRIDE_AUTOCONTRAST = False
         self.OVERRIDE_AUTOCONTRAST_LIMS = (0, 65535)
 
-
     def __eq__(self, other):
         if isinstance(other, Node):
             return self.id == other.id
         return False
 
     def clear_flags(self):
-        for a in self.connectable_attributes:
+        for a in self.connectable_attributes.values():
             a.clear_flags()
         if self.any_change and not cfg.focused_node == self:
             cfg.any_change = True
@@ -92,9 +95,9 @@ class Node:
         self.NODE_GAINED_FOCUS = False
 
     def render_start(self):
-        if self.use_roi == False:
+        if not self.use_roi:
             self.roi = [0, 0, 0, 0]
-        ## render the node window
+        # render the node window
         imgui.set_next_window_position(self.position[0], self.position[1], imgui.ALWAYS)
         imgui.set_next_window_size_constraints((self.size, 50), (self.size, 1000))
         imgui.push_style_color(imgui.COLOR_PLOT_HISTOGRAM, *self.colour)
@@ -174,17 +177,17 @@ class Node:
                 imgui.close_current_popup()
             # _duplicate, _ = imgui.menu_item("Duplicate node")
             # if _duplicate:
-            #     duplicate_node = Node.create_node_by_type(self.type) ## TODO FIX
+            #     duplicate_node = Node.create_node_by_type(self.type)
             #     duplicate_node.position = [self.position[0] + 10, self.position[1] + 10]
             #     cfg.set_active_node(duplicate_node)
             _delete, _ = imgui.menu_item("Delete node")
             if _delete:
                 self.delete()
             _reset, _ = imgui.menu_item("Reset node")
-            # if _reset:
-            #     new_node = Node.create_node_by_type(self.type) ## TODO FIX
-            #     new_node.position = self.position
-            #     self.delete()
+            if _reset:
+                new_node = Node.create_node_by_type(self.type)
+                self.params = copy.deepcopy(new_node.params)
+                new_node.delete()
             if imgui.begin_menu("Set node-specific LUT"):
                 for lut in ["auto"] + settings.lut_names:
                     _lut, _ = imgui.menu_item(lut)
@@ -279,13 +282,67 @@ class Node:
                 cfg.set_active_node(None, True)
             if cfg.active_node == self:
                 cfg.set_active_node(None)
+                cfg.active_connector_parent_node = None
+                cfg.active_node = None
+                cfg.active_connector = None
             if self in cfg.nodes:
                 cfg.nodes.remove(self)
-            for attribute in self.connectable_attributes:
+            for attribute in self.connectable_attributes.values():
                 attribute.delete()
         except Exception as e:
             cfg.set_error(e, "Problem deleting node\n"+str(e))
 
+    def to_dict(self):
+        node_dict = dict()
+        node_dict["title"] = self.title
+        node_dict["id"] = self.id
+        node_dict["position"] = self.position
+        node_dict["use_roi"] = self.use_roi
+        node_dict["roi"] = self.roi
+        node_dict["lut"] = self.lut
+        m_attributes = list()
+        for attribute_name in self.connectable_attributes:
+            attribute = self.connectable_attributes[attribute_name]
+            attribute_dict = dict()
+            attribute_dict["name"] = attribute_name
+            attribute_dict["type"] = attribute.type
+            attribute_dict["title"] = attribute.title
+            attribute_dict["direction"] = attribute.direction
+            attribute_dict["allowed_partner_types"] = attribute.allowed_partner_types
+            attribute_dict["id"] = attribute.id
+            m_attributes.append(attribute_dict)
+        node_dict["attributes"] = m_attributes
+        node_dict["params"] = self.params
+        return node_dict
+
+    def from_dict(self, node_dict):
+        if node_dict["title"] != self.title:
+            raise Exception(f'Node of type {self.title} was given a parameter dictionary for a node of type {node_dict["title"]}')
+        self.title = node_dict["title"]
+        self.id = node_dict["id"]
+        self.position = node_dict["position"]
+        self.use_roi = node_dict["use_roi"]
+        self.roi = node_dict["roi"]
+        self.lut = node_dict["lut"]
+        self.params = node_dict["params"]
+        self.connectable_attributes = dict()
+        for attribute_dict in node_dict["attributes"]:
+            attribute = ConnectableAttribute(attribute_dict["type"], attribute_dict["direction"], self, attribute_dict["allowed_partner_types"])
+            attribute.id = attribute_dict["id"]
+            attribute.title = attribute_dict["title"]
+            self.connectable_attributes[attribute_dict["name"]] = attribute
+
+    @staticmethod
+    def parallel_process(function, arglist):
+        print(f"Starting parallel process with joblib.\nfn: {function}\nargs: {arglist}")
+        print(f"n_cpus = {cfg.n_cpus}, mmap_mode = {settings.joblib_mmmode}")
+        retvals = Parallel(n_jobs=cfg.n_cpus, mmap_mode=settings.joblib_mmmode)(delayed(function)(i) for i in arglist)
+        return retvals
+
+    @staticmethod
+    def gen_temp_dir_name():
+        return str(datetime.datetime.now().strftime('%Y%m%d%H%M%S%f'))
+
     @staticmethod
     def tooltip(text):
         if imgui.is_item_hovered():
@@ -315,14 +372,17 @@ class Node:
         :return: node: node with type TYPE_LOAD_DATA
         """
         def get_any_incoming_node(_node):
-            for attribute in _node.connectable_attributes:
+            print(_node)
+            for attribute in _node.connectable_attributes.values():
                 if attribute.direction == ConnectableAttribute.INPUT:
+                    print("input")
                     return attribute.get_incoming_node()
 
         if node.NODE_IS_DATA_SOURCE:
             return node
         try:
             source = get_any_incoming_node(node)
+            print(source)
             while not isinstance(source, NullNode):
                 if source.NODE_IS_DATA_SOURCE:
                     return source
@@ -373,22 +433,25 @@ class Node:
     def on_gain_focus(self):
         self.NODE_GAINED_FOCUS = True
 
-    def pre_save(self):
+    def pre_pickle(self):
         cfg.pickle_temp = dict()
         self.last_index_requested = -1
         self.last_frame_returned = None
         cfg.pickle_temp["profiler_time"] = self.profiler_time
-        self.pre_save_impl()
+        self.pre_pickle_impl()
 
-    def pre_save_impl(self):
+    def pre_pickle_impl(self):
         pass
 
-    def post_save(self):
+    def post_pickle(self):
         self.profiler_time = cfg.pickle_temp["profiler_time"]
-        self.post_save_impl()
+        self.post_pickle_impl()
         cfg.pickle_temp = dict()
 
-    def post_save_impl(self):
+    def post_pickle_impl(self):
+        pass
+
+    def on_load(self):
         pass
 
 
@@ -464,8 +527,6 @@ class ConnectableAttribute:
         if allowed_partner_types is not None:
             self.allowed_partner_types = allowed_partner_types
 
-        parent.connectable_attributes.append(self)
-
     def __eq__(self, other):
         if type(self) is type(other):
             return self.id == other.id
